package org.jxul.css;

import java.io.*;
import java.util.*;

/**
 * A class to parse CSS files
 */
public class CssParser
{
    public static final boolean debug;
    
    static
    {
        // this variation works in both applets and applications
        boolean temp = false;
        try
        {
            String sdebug = System.getProperty("debug");
            if (sdebug != null && sdebug.equalsIgnoreCase("true"))
                temp = true;
            else
                temp = false;
        } catch (Exception e)
          { }
        debug = temp;
    }

    String cssFile;
    CssLexer lexer;
    
    public CssParser(String filename) throws IOException
    {
        cssFile = filename;
        lexer = new CssLexer(cssFile);
    }
    
    public StyleSheet parse() throws IOException
    {
        StyleSheet sheet = null;
        Style [] result = null;
        Vector tempVec = new Vector();
        
        Style currentStyle = null;
        
        // get all the tokens
        CssToken tok = null;
                
        while ( (tok = lexer.nextToken()) != null)
        {
            if (debug) System.out.println("Token: " + tok);
            switch (tok.getType())
            {
                case CssToken.TOK_SELECTOR:
                        if (currentStyle != null)
                            tempVec.add(currentStyle);                            
                        currentStyle = new Style(tok.getValue());
                    break;
                case CssToken.TOK_STATEMENT:
                        if (currentStyle != null)
                        {
                            if (debug) System.out.println("currentSelector: " + currentStyle.getSelector());
                            currentStyle.addStatement(tok.getValue());                            
                        }
                        else
                            System.out.println("Error: Statement without a selector");
                    break;
                default:
                    throw new RuntimeException("Unknown token type: " + tok.getType());
            }                    
        }        

        if (tempVec.size() > 0)
        {
            result = new Style[tempVec.size()];
            tempVec.copyInto(result);
        }
        
        if (result != null)
        {
            sheet = new StyleSheet(cssFile, result);
        }
        
        return sheet;
    }
        
    /** For Unit testing. */
    public static void main(String args[])
    {
        try
        {
            if (args.length < 1)
            {
                System.out.println("USAGE: java org.jxul.css.CssParser cssfile");
                System.exit(1);
            }
            
            String filename = args[0];
            
            // insure the file exists
            File f = new File(filename);
            
            if (f.exists())
            {
                // lex the file
                CssParser parser = new CssParser(filename);
                StyleSheet sheet = parser.parse();
                Style [] styles = sheet.getStyles();
                for (int i=0; i < styles.length; i++)
                {
                    System.out.println("Style[" + i + "]" + styles[i]);    
                }
            } 
            else
                System.out.println("Filename: " + filename + " does not exist.");
        } catch (Throwable t)
          {
            t.printStackTrace();
          }
    }
}
