/** MenuResource.java */
package org.jxul.demo;

import javax.swing.*;
import java.io.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import java.awt.event.*;
import org.jxul.framework.*;

/**
 * Class to test the XulResourceFactory with a 
 * Menu.
 */
public class MenuResource extends JFrame implements ActionListener
{
    public MenuResource(Document fragment)
    {
        // create our menu
        try
        {
            JMenuBar mb = (JMenuBar) XulResourceFactory.createComponent(fragment, this);
            this.setJMenuBar(mb);
            
            this.setVisible(true);
            this.pack();
            
            this.addWindowListener(new WindowAdapter()
                                   {
                                        public void windowClosing(WindowEvent we)
                                        { System.exit(0); }
                                   });
            
        } catch (Exception e)
          {
            e.printStackTrace();  
            System.exit(1);
          }
    }
   
    public void actionPerformed(ActionEvent evt)
    {
        String command = evt.getActionCommand();
        System.out.println("Event: " + command);
        if (command.equals("Exit"))
        {
            System.exit(0);    
        }
    }
    
    public static void main(String args[])
    {
        if (args.length < 1)
        {
            System.out.println("USAGE: java org.jxul.demo.MenuResource xulResourceFragment");
            System.exit(1);
        }
        
        try
        {
            String curDir = System.getProperty("user.dir");
            
            String inputFile = args[0];
            File f = new File(inputFile);

            if (f.exists())
            {
                // ensure document is well-formed
                DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
                DocumentBuilder dbldr = factory.newDocumentBuilder();
		
        		//note that we need to do something when there are chrome: references
                //in the DTD
                Document doc = dbldr.parse(new File(inputFile));

                System.out.println("Created org.w3c.Document of Fragment.");
                
                MenuResource mr = new MenuResource(doc);
            }
            else
                System.out.println("File: " + args[0] + " does not exist. Aborting.");
        } catch (Throwable t)
          {
            t.printStackTrace();
          }
    }
}