/* XulBox.java */
package org.jxul.framework;

import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import org.w3c.dom.*;

import org.jxul.util.*;
import org.jxul.css.*;

/**
 * Title:        JXul
 * Description:  Open Source project to interface Java and the
 * Java platform to the eXtensible User-interface Language (XUL).
 * Copyright:    Copyright (c) 2001
 * Company:      Open Source
 * @author Michael C. Daconta, Kevin T. Smith, Seor Vaughn Bullard
 * @version 0.1
 *
 * A XUL Box. (similar to a JPanel)
 */

public class XulBox extends JPanel
{
    public static final boolean debug;

    static
    {
        // this variation works in both applets and applications
        boolean temp = false;
        try
        {
            String sdebug = System.getProperty("debug");
            if (sdebug != null && sdebug.equalsIgnoreCase("true"))
                temp = true;
            else
                temp = false;
        } catch (Exception e)
          { }
        debug = temp;
    }

    protected Element boxElement;
    // attributes
    String style;
    String align;
    String valign;
    String _class;
    String id;
    String flex;
    String width;
    String height;
    String autostretch;
    String orient;
    String oncommand;

    Style cssStyle;
    Hashtable peerMap;

    public XulBox(Element boxElement, Hashtable peerMap) throws XulFormatException
    {
        this.boxElement = boxElement;
        this.peerMap = peerMap;

        // TBD: work out how to modify for all these attributes
        // alignment of the children
        Attr alignAttr = boxElement.getAttributeNode(XulSyntax.BOX_ATTR_ALIGN);
        if (alignAttr != null)
        {
            align = alignAttr.getNodeValue();

            // set a Box Layout (if this is vertical or horizontal)
            if (align.equals(XulSyntax.BOX_ATTR_ALIGN_VERTICAL))
                this.setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));
            else
                this.setLayout(new BoxLayout(this,BoxLayout.X_AXIS));
        }

        if (getLayout() == null)
            this.setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));

        Attr valignAttr = boxElement.getAttributeNode(XulSyntax.BOX_ATTR_VALIGN);

        Attr _classAttr = boxElement.getAttributeNode(XulSyntax.BOX_ATTR_CLASS);

        Attr idAttr = boxElement.getAttributeNode(XulSyntax.BOX_ATTR_ID);

        Attr flexAttr = boxElement.getAttributeNode(XulSyntax.BOX_ATTR_FLEX);

        Attr widthAttr = boxElement.getAttributeNode(XulSyntax.BOX_ATTR_WIDTH);

        Attr heightAttr = boxElement.getAttributeNode(XulSyntax.BOX_ATTR_HEIGHT);

        Attr autostretchAttr = boxElement.getAttributeNode(XulSyntax.BOX_ATTR_AUTOSTRETCH);

        Attr orientAttr = boxElement.getAttributeNode(XulSyntax.BOX_ATTR_ORIENT);
        if (orientAttr != null)
        {
            orient = orientAttr.getNodeValue();

            // set a Box Layout (if this is vertical or horizontal)
            if (orient.equals(XulSyntax.BOX_ATTR_ORIENT_VERTICAL))
                this.setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));
            else
                this.setLayout(new BoxLayout(this,BoxLayout.X_AXIS));
        }

        Attr oncommandAttr = boxElement.getAttributeNode(XulSyntax.BOX_ATTR_ONCOMMAND);

        Attr styleAttr = boxElement.getAttributeNode(XulSyntax.BOX_ATTR_STYLE);
        if (styleAttr != null)
        {
            // styling overriding the .css stylesheet
            style = styleAttr.getNodeValue();

            if (style != null && style.length() > 0)
            {
                // TBD:
                // create a CSS Style
                cssStyle = new Style(Style.NONAME,style);

                // apply it
                cssStyle.apply(this);
            }
        }

        processChildren();
    } // end of XulBox Constructor

    public void processChildren() throws XulFormatException
    {
        processChildren(boxElement);
    }

    public void processChildren(Element theElement) throws XulFormatException
    {
        // should have widget children
        if (theElement.hasChildNodes())
        {
            /* children should be a set of widget elements
               or more boxes. */
            NodeList children = theElement.getChildNodes();

            // iterate through each child, if a menu - create it
            int childCount = children.getLength();
            for (int i=0; i < childCount; i++)
            {
                // get the child node
                Node child = children.item(i);

	        if (debug)
                    System.out.println("Child[" + i + "]" +
                    child.getNodeName() + "-" + child.getNodeValue());

                // switch on the type of the child
                String childName = child.getNodeName();

		if (childName.equals(XulSyntax.ELEMENT_HTMLIFRAME) ||
		    childName.equals(XulSyntax.ELEMENT_IFRAME))
		{
		    XulIframe iframe = new XulIframe((Element)child);
		    this.add(iframe);
		    peerMap.put(child, iframe);
		}
		else
		if (childName.indexOf("html")==0)
		{
		   //hack for mini html inside XUL -yuck!
		    XulHTMLText htm = new XulHTMLText((Element)child);
		    this.add(htm);
		    peerMap.put(child, htm);
		}
		else
		if (childName.equals(XulSyntax.ELEMENT_TITLE))
		{
		    XulTitle title = new XulTitle((Element)child);
                    //title puts a titledborder around us..
		    this.setBorder(title);
		    peerMap.put(child, title);
		}
		else
		if (childName.equalsIgnoreCase(XulSyntax.ELEMENT_TREE))
		{
		    XulTree tree = new XulTree((Element)child, peerMap);
		    peerMap.put(child,tree);
		    this.add(tree);
		}
		else
		if (childName.equals(XulSyntax.ELEMENT_BOX))
		{
		    XulBox box = new XulBox((Element)child, peerMap);
		    if (cssStyle != null)
                         cssStyle.apply(box);

                    this.add(box);

                    // add to peerMap
		    peerMap.put(child, box);
		}
		else
                if (childName.equals(XulSyntax.ELEMENT_TITLEDBOX))
		{
		    XulTitledBox tbox = new XulTitledBox((Element)child,peerMap);
		    if (cssStyle != null)
                        cssStyle.apply(tbox);
		    System.out.println("adding Titledbox to Xulbox");
                    this.add(tbox);

                    // add to peerMap
                   peerMap.put(child, tbox);
		}
		else // Added by Seor Vaughn - 8 JUL 2001 - Was missed originally
		if (childName.equals(XulSyntax.ELEMENT_GROUPBOX))
                {
                    XulGroupBox gbox = new XulGroupBox((Element)child,peerMap);
                    if (cssStyle != null)
                        cssStyle.apply(gbox);
                    System.out.println("adding Groupbox to Xulbox");
                    this.add(gbox);

                    // add to peerMap
                   peerMap.put(child, gbox);
                }
		else
                if (childName.equals(XulSyntax.ELEMENT_BUTTON))
                {
                    XulButton button = new XulButton((Element)child);
                    if (cssStyle != null)
                        cssStyle.apply(button);
                    this.add(button);

                    // add to peerMap
                    peerMap.put(child, button);
                }
                else if (childName.equals(XulSyntax.ELEMENT_PROGRESSMETER))
                {
                    XulProgressMeter xpm = new XulProgressMeter((Element)child,peerMap);
                    if (cssStyle != null)
                        cssStyle.apply(xpm);

                    this.add(xpm);

                    // add to peerMap
                    peerMap.put(child, xpm);
                }
                else if (childName.equals(XulSyntax.ELEMENT_CHECKBOX))
                {
                    XulCheckBox checkBox = new XulCheckBox((Element)child);
                    if (cssStyle != null)
                        cssStyle.apply(checkBox);

                    this.add(checkBox);

                    // add to peerMap
                    peerMap.put(child, checkBox);
                }
                else if (childName.equals(XulSyntax.ELEMENT_RADIOGROUP))
                {
                    XulRadioGroup radioGroup = new XulRadioGroup((Element)child, cssStyle);
                    if (cssStyle != null)
                        cssStyle.apply(radioGroup);

                    this.add(radioGroup);

                    // add to peerMap
                    peerMap.put(child, radioGroup);
                }
                else if (childName.equals(XulSyntax.ELEMENT_TEXT))
                {
                    XulText text = new XulText((Element)child);

                    // add this to the menu bar
                    this.add(text);

                    // add to peerMap
                    peerMap.put(child, text);
                }
		        else if (childName.equals(XulSyntax.ELEMENT_LABEL))
                {
                    XulLabel label = new XulLabel((Element)child);

                    //Okay, if this is a XulTitledBox, and label is the
                    //first element, then that's what our border will be
                    //and we shall not add it.
                    if (boxElement.getNodeName().equals(XulSyntax.ELEMENT_TITLEDBOX) && i==0)
                    {
                       TitledBorder border = new TitledBorder(label.getText());
            		   this.setBorder(border);
                    }
                    else
                    {

                       // add this
                       this.add(label);

                       // add to peerMap
                       peerMap.put(child, label);
                }
                }
                else if (childName.equals(XulSyntax.ELEMENT_TABBOX))
                {
                    // create a XulTabControl
                    XulTabBox tabbox = new XulTabBox((Element)child, peerMap);

                    // add it
                    this.add(tabbox);

                    // add to peerMap
                    peerMap.put(child, tabbox);
                }
                else if (childName.equals(XulSyntax.ELEMENT_HBOX))
                {
                    // create a XulHBox
                    XulHBox hbox = new XulHBox((Element)child, peerMap);

                    // add it
                    this.add(hbox);

                    // add to peerMap
                    peerMap.put(child, hbox);
                }
                else if (childName.equals(XulSyntax.ELEMENT_VBOX))
                {
                    // create a XulVBox
                    XulVBox vbox = new XulVBox((Element)child, peerMap);

                    // add it
                    this.add(vbox);

                    // add to peerMap
                    peerMap.put(child, vbox);
                }
                else if (childName.equals(XulSyntax.ELEMENT_IMAGE))
                {
                    if (debug) System.out.println("adding an image...");
                    // create a XulImage
                    XulImage image = new XulImage((Element)child);

                    // add it
                    this.add(image);

                    // add to peerMap
                    peerMap.put(child, image);
                }
                else if (childName.equals(XulSyntax.ELEMENT_GRID))
                {
                    // create a XulImage
                    XulGrid grid = new XulGrid((Element)child, peerMap);

                    // add it
                    this.add(grid);

                    // add to peerMap
                    peerMap.put(child, grid);
                }
                else if (childName.equals(XulSyntax.ELEMENT_STACK))
                {
                    // create a XulStack
                    XulStack stack = new XulStack((Element)child, peerMap);

                    // add it
                    this.add(stack);

                    // add to peerMap
                    //peerMap.put(child, stack);
                }
                else if (childName.equals(XulSyntax.ELEMENT_BULLETIN_BOARD))
                {
                    // create a XulBulletinBoard
                    XulBulletinBoard bb = new XulBulletinBoard((Element)child, peerMap);

                    // add it
                    this.add(bb);

                    // add to peerMap
                    peerMap.put(child, bb);
                }
                else if (childName.equals(XulSyntax.ELEMENT_TEXTFIELD))
                {
                    // get a JTextField or JTextArea
                    JComponent jc = XulTextField.getTextField((Element)child);

                    // add it
                    this.add(jc);

                    // add to peerMap
                    peerMap.put(child, jc);
                }
		else if (childName.equals(XulSyntax.ELEMENT_TEXTBOX))
                {
                    // get a JTextField or JTextArea
                    JComponent jc = XulTextBox.getTextField((Element)child);

                    // add it
                    this.add(jc);

                    // add to peerMap
                    peerMap.put(child, jc);
                }
		else if (childName.equals(XulSyntax.ELEMENT_TOOLBOX))
		{
		     XulToolBox tb = new XulToolBox((Element)child, peerMap);
		     this.add(tb);
		     peerMap.put(child, tb);
		}
                else
                {
                    System.out.println("*TBD*:Unhandled XUL element: " + childName);
                }

                /*
                else if (childName.equals(XulSyntax.ELEMENT_MENU))
                {
                    XulMenu menu = new XulMenu((Element)child);

                    // add this to the menu bar
                    this.add(menu);

                    // add to peerMap
                    peerMap.put(child, menu);
                }
                */

            } // end of for (all children)
        } // end of if (children exist)
    }

    /** for unit testing. */
    public static void main(String args[])
    {
        try
        {

        } catch (Throwable t)
          {
            t.printStackTrace();
          }
    }
}
