/* XulButton.java */
package org.jxul.framework;

import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import java.net.*;

import org.w3c.dom.*;

import org.jxul.util.*;
import org.jxul.css.*;
import org.jxul.js.*;

/**
 *
 * <P>
 *   A XUL Button 
 * </P>
 * @author Michael C. Daconta
 * @version 0.1
 */
 
 /*
  * A XUL Button.
  */
public class XulButton extends JButton implements ActionListener
{
    public static final boolean debug;
    
    static
    {
        // this variation works in both applets and applications
        boolean temp = false;
        try
        {
            String sdebug = System.getProperty("debug");
            if (sdebug != null && sdebug.equalsIgnoreCase("true"))
                temp = true;
            else
                temp = false;
        } catch (Exception e)
          { }
        debug = temp;
    }

    private XulJsEngine jsEngine = null;    
    private Element buttonElement;
    private String jsFunction = null;

    // attributes
    String style;
    String value;
    String _class;
    String onclick;
    String oncommand;
    String id;
    String tooltip;
    String _default;
    String disabled;
    String toggled;
    String src;
    String crop;

    Style cssStyle;

    public XulButton(Element buttonElement)
    {
        this.buttonElement = buttonElement;
        
        // TBD: process all attributes

        Attr valueAttr = buttonElement.getAttributeNode(XulSyntax.BUTTON_ATTR_LABEL);
        if (valueAttr != null)
        {
            value = valueAttr.getNodeValue();
            this.setText(value);
        }
        
        Attr _classAttr = buttonElement.getAttributeNode(XulSyntax.BUTTON_ATTR_CLASS);
        Attr onclickAttr = buttonElement.getAttributeNode(XulSyntax.BUTTON_ATTR_ONCLICK);
        Attr oncommandAttr = buttonElement.getAttributeNode(XulSyntax.BUTTON_ATTR_ONCOMMAND);
        Attr idAttr = buttonElement.getAttributeNode(XulSyntax.BUTTON_ATTR_ID);
        Attr tooltipAttr = buttonElement.getAttributeNode(XulSyntax.BUTTON_ATTR_TOOLTIP);
        Attr _defaultAttr = buttonElement.getAttributeNode(XulSyntax.BUTTON_ATTR_DEFAULT);
        Attr disabledAttr = buttonElement.getAttributeNode(XulSyntax.BUTTON_ATTR_DISABLED);
        Attr toggledAttr = buttonElement.getAttributeNode(XulSyntax.BUTTON_ATTR_TOGGLED);
        Attr srcAttr = buttonElement.getAttributeNode(XulSyntax.BUTTON_ATTR_SRC);
        if (srcAttr != null)
        {
            try
            {
                // button has an image associated with it.
                src = srcAttr.getNodeValue();
                // check if this has a colon (i.e. a URL)
                ImageIcon icon = null;
                if (src.indexOf(':') >= 0)
                {
                    URL url = new URL(src);
                    icon = new ImageIcon(url);
                }
                else
                    icon = new ImageIcon(src);
                
                this.setIcon(icon);
            } catch (Exception e)
              {
                e.printStackTrace();
                System.out.println("Unable to use src attribute: " + src);
              }
        }
        
        Attr cropAttr = buttonElement.getAttributeNode(XulSyntax.BUTTON_ATTR_CROP);

        Attr styleAttr = buttonElement.getAttributeNode(XulSyntax.BOX_ATTR_STYLE);
        if (styleAttr != null)
        {
            // styling overriding the .css stylesheet
            style = styleAttr.getNodeValue();
            
            if (style != null && style.length() > 0)
            {
                // TBD:
                // create a CSS Style
                cssStyle = new Style(Style.NONAME,style);
                
                // apply it
                try
                {
                    cssStyle.apply(this);
                } catch (XulFormatException xfe)
                  {
                    xfe.printStackTrace();  
                  }
            }
        }
	if (oncommandAttr != null)
	{
		jsFunction = oncommandAttr.getNodeValue();
		addActionListener(this);
	}
		
    }
    public void actionPerformed(ActionEvent e)
    {
	if (jsFunction != null)
	{
		System.out.println("Need to execute " + jsFunction + " here!");
		jsEngine = XulJsEngine.getInstance();

		try	
		{
			jsEngine.executeFunction(jsFunction);	
		}
		catch (Exception jsexc)
		{
			jsexc.printStackTrace();
		}
	}
    }    
    /** for unit testing. */
    public static void main(String args[])
    {
        try
        {
            
        } catch (Throwable t)
          {
            t.printStackTrace();
          }
    }
}
