/* XulMenuBar.java */
package org.jxul.framework;

import java.io.*;
import java.util.*;
import javax.swing.*;
import java.awt.event.*;

import org.w3c.dom.*;

import org.jxul.util.*;

/**
 * A XUL MenuBar.
 */
public class XulMenuBar extends JMenuBar
{
    public static final boolean debug;
    
    static
    {
        // this variation works in both applets and applications
        boolean temp = false;
        try
        {
            String sdebug = System.getProperty("debug");
            if (sdebug != null && sdebug.equalsIgnoreCase("true"))
                temp = true;
            else
                temp = false;
        } catch (Exception e)
          { }
        debug = temp;
    }
    
    private Element menuBarElement;
    // attributes
    String style;
    String chromeclass;
    String _class;
    String id;
    String flex;
    String align;
    String orient;
    String autostretch;
    Hashtable peerMap;

    public XulMenuBar(Element menuBarElement, Hashtable peerMap) throws XulFormatException
    {
        this(menuBarElement, peerMap, null);    
    }
    
    public XulMenuBar(Element menuBarElement, Hashtable peerMap, ActionListener al) throws XulFormatException
    {
        this.menuBarElement = menuBarElement;
        this.peerMap = peerMap;
        
        // TBD: work out how to modify for all these attributes
        
        Attr chromeclassAttr = menuBarElement.getAttributeNode(XulSyntax.MENUBAR_ATTR_CHROMECLASS);
        if (chromeclassAttr != null)
            chromeclass = chromeclassAttr.getNodeValue();
            
        Attr _classAttr = menuBarElement.getAttributeNode(XulSyntax.MENUBAR_ATTR_CLASS);
        if (_classAttr != null)
            _class = _classAttr.getNodeValue();
            
        Attr idAttr = menuBarElement.getAttributeNode(XulSyntax.MENUBAR_ATTR_ID);
        if (idAttr != null)
            id = idAttr.getNodeValue();
            
        Attr flexAttr = menuBarElement.getAttributeNode(XulSyntax.MENUBAR_ATTR_FLEX);
        if (flexAttr != null)
            flex = flexAttr.getNodeValue();
            
        Attr alignAttr = menuBarElement.getAttributeNode(XulSyntax.MENUBAR_ATTR_ALIGN);
        if (alignAttr != null)
            align = alignAttr.getNodeValue();
            
        Attr orientAttr = menuBarElement.getAttributeNode(XulSyntax.MENUBAR_ATTR_ORIENT);
        if (orientAttr != null)
            orient = orientAttr.getNodeValue();
            
        Attr autostretchAttr = menuBarElement.getAttributeNode(XulSyntax.MENUBAR_ATTR_AUTOSTRETCH);
        if (autostretchAttr != null)
            autostretch = autostretchAttr.getNodeValue();

        // should have menu children
        if (menuBarElement.hasChildNodes())
        {
            // children should be a set of Menu elements
            NodeList children = menuBarElement.getChildNodes();
            
            // iterate through each child, if a menu - create it
            int childCount = children.getLength();
            for (int i=0; i < childCount; i++)
            {
                // get the child node
                Node child = children.item(i);
                
                if (child.getNodeName().equals(XulSyntax.ELEMENT_MENU))
                {
                    XulMenu menu = new XulMenu((Element)child, peerMap, al);
                    
                    // add this to the menu bar
                    this.add(menu);

                    // add to peerMap
                    peerMap.put(child, menu);
                }                
            } // end of for (all children)
        } // end of if (children exist)       
    } // end of XulMenuBar Constructor
    
    /** for unit testing. */
    public static void main(String args[])
    {
        try
        {
            
        } catch (Throwable t)
          {
            t.printStackTrace();
          }
    }
}
