package org.jxul.framework;

import javax.swing.*;
import org.w3c.dom.*;
import java.awt.event.*;
import java.util.*;

/**
 * Class to generate JComponents from XUL snippets.
 * Components supported:
 * 1. MenuBar
 * 2. ToolBar
 * 
 * @version 0.1
 */
public class XulResourceFactory
{
    public static final boolean debug;
    
    static
    {
        // this variation works in both applets and applications
        boolean temp = false;
        try
        {
            String sdebug = System.getProperty("debug");
            if (sdebug != null && sdebug.equalsIgnoreCase("true"))
                temp = true;
            else
                temp = false;
        } catch (Exception e)
          { }
        debug = temp;
    }
    
    /**
     * Method to create a Component from a XUL Snippet and programmatically 
     * register an event listener.
     * @param fragment The document fragment.  The name of the component MUST
     *                 be the root element of the fragment.
     * @param l The event listener to connect to the component.
     */
    public static JComponent createComponent(Document fragment, EventListener l) throws XulResourceException, XulFormatException
    {
        // get the root element
        Element elem = fragment.getDocumentElement();
        String elemName = elem.getNodeName();
        if (debug) System.out.println("Element is <" + elem.getNodeName() + ">");        
        
        // temp -- create Xul Widget constructors that don't use a peerMap
        Hashtable peerMap = new Hashtable(); 
        
        if (elemName.equals(XulSyntax.ELEMENT_MENUBAR))
        {
            // create a XulMenuBar
            JMenuBar menuBar = new XulMenuBar((Element)elem, peerMap, (ActionListener) l);
            
            return menuBar;
        }
        else if (elemName.equals(XulSyntax.ELEMENT_TOOLBAR))
        {
            // create the tool bar
            JToolBar toolbar = new XulToolBar((Element) elem, peerMap, (ActionListener) l);
            
            return toolbar;
        }
        else
        {
            throw new XulResourceException("Unknown element: " + elem.getNodeName());    
        }
    }
}
