/* XulToolBox.java */
package org.jxul.framework;

import java.io.*;
import java.util.*;
import javax.swing.*;

import org.w3c.dom.*;

import org.jxul.util.*;
import org.jxul.css.*;

/**
 * <P>A XUL ToolBox</P>
 * @author Kevin T. Smith
 * @version 0.1
 */
public class XulToolBox extends JPanel
{
    public static final boolean debug;
    
    static
    {
        // this variation works in both applets and applications
        boolean temp = false;
        try
        {
            String sdebug = System.getProperty("debug");
            if (sdebug != null && sdebug.equalsIgnoreCase("true"))
                temp = true;
            else
                temp = false;
        } catch (Exception e)
          { }
        debug = temp;
    }
    
    protected Element toolboxElement;
    // attributes
    String _class;
    String id;
    String chromeClass;
    String orient;
    String autostretch;
    String style;
    
    Style cssStyle;
    Hashtable peerMap;
    
   
    public XulToolBox(Element toolboxElement, Hashtable peerMap) throws XulFormatException
    {
        this.toolboxElement = toolboxElement;
        this.peerMap = peerMap;
        
        if (getLayout() == null)
            this.setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));
        
        Attr _classAttr = toolboxElement.getAttributeNode(XulSyntax.TOOLBOX_ATTR_CLASS);
        if (_classAttr != null)
		_class = _classAttr.getNodeValue();
 
        Attr idAttr = toolboxElement.getAttributeNode(XulSyntax.TOOLBOX_ATTR_ID);
        if (idAttr != null)
        	id = idAttr.getNodeValue();       
        Attr autostretchAttr = toolboxElement.getAttributeNode(XulSyntax.TOOLBOX_ATTR_AUTOSTRETCH);
        if (autostretchAttr != null)
     		autostretch = autostretchAttr.getNodeValue();
 
        Attr orientAttr = toolboxElement.getAttributeNode(XulSyntax.TOOLBOX_ATTR_ORIENT);

        if (orientAttr != null)
        {
            orient = orientAttr.getNodeValue();
            
            // set a Box Layout (if this is vertical or horizontal)
            if (orient.equals(XulSyntax.TOOLBOX_ATTR_ORIENT_VERTICAL))
                this.setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));
            else
                this.setLayout(new BoxLayout(this,BoxLayout.X_AXIS));                
        }
        
        Attr styleAttr = toolboxElement.getAttributeNode(XulSyntax.TOOLBOX_ATTR_STYLE);
        if (styleAttr != null)
        {
            // styling overriding the .css stylesheet
            style = styleAttr.getNodeValue();
            
            if (style != null && style.length() > 0)
            {
                // TBD:
                // create a CSS Style
                cssStyle = new Style(Style.NONAME,style);
                
                // apply it
                cssStyle.apply(this);
            }
        }
        
        processChildren();        
    } // end of XulToolBox Constructor

    public void processChildren() throws XulFormatException
    {
        // should have widget children
        if (toolboxElement.hasChildNodes())
        {
            /* only children should be a toolbar */
            NodeList children = toolboxElement.getChildNodes();
            
            // iterate through each child, if a toolbar - create it

            int childCount = children.getLength();
            for (int i=0; i < childCount; i++)
            {
                // get the child node
                Node child = children.item(i);
                if (debug)
                     System.out.println("Child[" + i + "]" +
                     child.getNodeName() + "-" + child.getNodeValue());
 
                // switch on the type of the child
                
                if (child.getNodeName().equals(XulSyntax.ELEMENT_TOOLBAR))
                {
                    XulToolBar toolbar = new XulToolBar((Element)child, peerMap);
                    if (cssStyle != null)
                        cssStyle.apply(toolbar);
                    this.add(toolbar);
                }
                
            } // end of for (all children)
        } // end of if (children exist)               
    }
    
    /** for unit testing. */
    public static void main(String args[])
    {
        try
        {
            
        } catch (Throwable t)
          {
            t.printStackTrace();
          }
    }
}
