/* XulTreeRow.java */
package org.jxul.framework;

import java.io.*;
import java.util.*;
import org.w3c.dom.*;
import org.jxul.util.*;

/**
 * A XUL TreeRow which is a row of values
 * It will have <treecells> as children
 */
public class XulTreeRow 
{
    public static final boolean debug;
    
    static
    {
        // this variation works in both applets and applications
        boolean temp = false;
        try
        {
            String sdebug = System.getProperty("debug");
            if (sdebug != null && sdebug.equalsIgnoreCase("true"))
                temp = true;
            else
                temp = false;
        } catch (Exception e)
          { }
        debug = temp;
    }

    //for use with treetable.XulTreeModel
    private String treerowText;  
    private Element treerowElement;
    private Vector rowItems = new Vector();

    // attributes
    String _class;
    String id;

    Hashtable peerMap;

    public XulTreeRow(Element treerowElement, Hashtable peerMap) throws XulFormatException
    {
        this.treerowElement = treerowElement; 
	this.peerMap = peerMap;

        Attr _classAttr = treerowElement.getAttributeNode(XulSyntax.ATTR_CLASS);
        Attr idAttr = treerowElement.getAttributeNode(XulSyntax.ATTR_ID);
        
        processChildren();
    } // end of XulTreeRow Constructor
        
    public void processChildren() throws XulFormatException
    {
        // should have widget children
        if (treerowElement.hasChildNodes())
        {
            NodeList children = treerowElement.getChildNodes();
         
            int childCount = children.getLength();
	    
            for (int i=0; i < childCount; i++)
            {
                // get the child node
                Node child = children.item(i);
                String childName = child.getNodeName(); 
                if (debug)
                    System.out.println("Child[" + i + "]" +
                    childName + "-" + child.getNodeValue());
 
                // switch on the type of the child
               
		if (childName.equalsIgnoreCase("treecell"))
		{
			XulTreeCell tc = new XulTreeCell((Element)child, peerMap);
			peerMap.put(tc, child);

			setText(tc.getValue());
			if (tc.isIndented())
			{	
				//figure out something here.
			}
			rowItems.add(tc.getValue());
		}
		else
                {
			if (debug)
				System.out.println("*TBD - " + childName);
                }
            } // end of for (all children)
        } // end of if (children exist)                       
    }

    //for treetable.XulTreeModel
    private void setText(String text)
    {
	treerowText = text;
    }
    public String getText()
    {
	return (treerowText);	
    }     
    public Vector getRowValues()
    {
	return (rowItems);
    }
    /** for unit testing. */
    public static void main(String args[])
    {
        try
        {
            
        } catch (Throwable t)
          {
            t.printStackTrace();
          }
    }
}
