/* XulApplication.java */
package org.jxul.xulrunner;

import java.io.*;
import java.util.*;
import java.net.*;
import java.awt.event.*;

import org.w3c.dom.*;
import org.mozilla.javascript.*;

import org.jxul.framework.*;
import org.jxul.util.*;
import org.jxul.css.*;
import org.jxul.js.*;

/**
 * A Xul Driven Application.
 */
public class XulApplication extends Thread
{
    public static final boolean debug;

    static
    {
        // this variation works in both applets and applications
        boolean temp = false;
        try
        {
            String sdebug = System.getProperty("debug");
            if (sdebug != null && sdebug.equalsIgnoreCase("true"))
                temp = true;
            else
                temp = false;
        } catch (Exception e)
          { }
        debug = temp;
    }

    Document xulDoc;

    /**
     * Mapping between Xul elements and their gui peers.
     * Necessary to map between the declarative description and
     * the implementation.  This will allow both processing of
     * stylesheets and targets for scripted actions.
     */
    Hashtable peerMap = new Hashtable();

    /**
     * Encapsulation of the doc and corresponding peers.
     */
    XulDom xulDom = new XulDom();

    StyleSheet styleSheet;

    Element root;
    XulWindow window;
    XulMenuBar menuBar;

    //Javascript engine
    XulJsEngine jsEngine = null;

    public XulApplication(Document xulDoc) throws XulFormatException
    {
        this.xulDoc = xulDoc;
        xulDom.setXulDoc(xulDoc);
        xulDom.setPeerMap(peerMap);

        // root element should be a window
        root = xulDoc.getDocumentElement();
        if (!root.getTagName().equals(XulSyntax.ELEMENT_WINDOW) && !root.getTagName().equals(XulSyntax.ELEMENT_OVERLAY))
            throw new XulFormatException("Root element must be a " + XulSyntax.ELEMENT_WINDOW + "or an " + XulSyntax.ELEMENT_OVERLAY);
    }

    public void run()
    {
        try
        {
            // TBD - check for xml-stylesheet processing instruction
            // if there -- parse it.
            NodeList siblings = xulDoc.getChildNodes();
            int siblingCount = siblings.getLength();
            // pi?
            for (int i=0; i < siblingCount; i++)
            {
                org.w3c.dom.Node sibling = siblings.item(i);
                int nType = sibling.getNodeType();
                if (nType == org.w3c.dom.Node.PROCESSING_INSTRUCTION_NODE)
                {
                    if (debug) System.out.println("DOC has a PI!!!");

                    String appName = sibling.getNodeName();

                    // check if a stylesheet
                    System.out.println("name: " + sibling.getNodeName());

                    if (appName.equals("xml-stylesheet"))
                    {
                        if (debug) System.out.println("This is a stylesheet!");

                        if (debug) System.out.println("value: " + sibling.getNodeValue());

                        // get attributes
                        String href = XmlUtil.extractAttributeValue(sibling.getNodeValue(), "href");
                        String type = XmlUtil.extractAttributeValue(sibling.getNodeValue(), "type");
                        if (debug) System.out.println("href = " + href + ",type = " + type);

			            //chrome: & jxul urls - something we need to think about..
                        href = XulUtil.translateURL(href);

                        if (href != null && type != null)
                        {
                            // css or xsl?
                            if (type.equals("text/css"))
                            {
                                // href or relative file?
                                if (href.indexOf(':') >= 0)
                                {
                                    try
                                    {
                                        URL url = new URL(href);

                                        System.out.println("TBD: get from URL:" + url.toString());

                                    } catch (MalformedURLException mfue)
                                      {
				        if (debug)
                                                mfue.printStackTrace(); //debug for kevin
                                        throw new XulFormatException("Invalid url for Image src attribute");
                                      }
                                }
                                else
                                {
                                    // get from File
                                    CssParser parser = new CssParser(href);
                                    styleSheet = parser.parse();
                                }
                            }
                        }
                    }
                }
            }

            /* render the Application
            - create the Xul Window */
            window = new XulWindow(root);

            // normalize the DOM tree
            DomUtil.normalizeDocument(root);

            /* now that we have a window,
            iterate through the child elements of the Window.
            The children will be widgets to add to the window. */
            if (root.hasChildNodes())
            {
                NodeList children = root.getChildNodes();

                int childCount = children.getLength();

                for (int i=0; i < childCount; i++)
                {
                    org.w3c.dom.Node child = children.item(i);

                    if (debug)
                        System.out.println("Child[" + i + "]" +
                        child.getNodeName() + "-" + child.getNodeValue());

                    String childElementName = child.getNodeName();

                    /* TBD: create an array of the node info and
                    Component created -- so that the created
                    widgets can be "styled" (if a stylesheet exists). */

                    // generate the widgets
		    if (childElementName.equalsIgnoreCase(XulSyntax.ELEMENT_TREE))
		    {
			XulTree tree = new XulTree((Element)child, peerMap);
			window.getContentPane().add(tree);
			peerMap.put(child, tree);
                    }
		    else
                    if (childElementName.equals(XulSyntax.ELEMENT_MENUBAR))
                    {
                        // create a XulMenuBar
                        menuBar = new XulMenuBar((Element)child, peerMap);

                        // set the menu bar
                        window.setJMenuBar(menuBar);
                    }
                    else if (childElementName.equals(XulSyntax.ELEMENT_BOX))
                    {
                        // create a XulBox
                        XulBox box = new XulBox((Element)child, peerMap);

                        // add it to the window
                        window.getContentPane().add(box);

                        // add to peerMap
                        peerMap.put(child, box);
                    }
                    else if (childElementName.equals(XulSyntax.ELEMENT_PROGRESSMETER))  // Added by Seor Vaughn - 8 JUL 2001
                    {
                        // create a XulProgressMeter
                        XulProgressMeter xpm = new XulProgressMeter((Element)child, peerMap);

                        // add it to the window
                        window.getContentPane().add(xpm);

                        // add to peerMap
                        peerMap.put(child, xpm);
                    }
                    else if (childElementName.equals(XulSyntax.ELEMENT_TITLEDBOX))
                    {
                        // create a XulTitledBox
                        XulTitledBox box = new XulTitledBox((Element)child, peerMap);

                        // add it to the window
                        window.getContentPane().add(box);

                        // add to peerMap
                        peerMap.put(child, box);
                    }
                    else if (childElementName.equals(XulSyntax.ELEMENT_VBOX))
                    {
                        // create a XulVBox
                        XulVBox vbox = new XulVBox((Element)child, peerMap);

                        // add it to the window
                        window.getContentPane().add(vbox);

                        // add to peerMap
                        peerMap.put(child, vbox);
                    }
                    else if (childElementName.equals(XulSyntax.ELEMENT_HBOX))
                    {
                        // create a XulHBox
                        XulHBox hbox = new XulHBox((Element)child, peerMap);

                        // add it to the window
                        window.getContentPane().add(hbox);

                        // add to peerMap
                        peerMap.put(child, hbox);
                    }
                    else if (childElementName.equals(XulSyntax.ELEMENT_TABBOX))
                    {
                        // create a XulTabBox
                        XulTabBox tabBox = new XulTabBox((Element)child, peerMap);

                        // add it to the window
                        window.getContentPane().add(tabBox);

                        // add to peerMap
                        peerMap.put(child, tabBox);
                    }
                    else if (childElementName.equals(XulSyntax.ELEMENT_GRID))
                    {
                        // create a XulGrid
                        XulGrid grid = new XulGrid((Element)child, peerMap);

                        // add it to the window
                        window.getContentPane().add(grid);

                        // add to peerMap
                        peerMap.put(child, grid);
                    }
                    else if (childElementName.equals(XulSyntax.ELEMENT_TOOLBOX))
                    {
                        XulToolBox toolBox = new XulToolBox((Element)child, peerMap);

                        window.getContentPane().add(toolBox);

                        // add to peerMap
                        peerMap.put(child, toolBox);
                    }
                    else if (childElementName.equals(XulSyntax.ELEMENT_TOOLBAR))
                    {
                        //question.. does a toolbar have to be a child of
                        //toolbox?
                        XulToolBar toolBar = new XulToolBar((Element)child, peerMap);
                        window.getContentPane().add(toolBar);

                        // add to peerMap
                        peerMap.put(child, toolBar);
                    }
                    // TBD: fix this -- JLayeredPane did not work correctly!!
                    else if (childElementName.equals(XulSyntax.ELEMENT_STACK))
                    {
                        // create a XulStack
                        XulStack stack = new XulStack((Element)child, peerMap);

                        // add it to the window
                        window.getContentPane().add(stack);

                        // add to peerMap
                        peerMap.put(child, stack);
                    }
                    /* -- Does not work correctly yet.  Should not occlude in a stack. --
                    else if (childElementName.equals(XulSyntax.ELEMENT_BULLETIN_BOARD))
                    {
                        // create a XulBulletinBoard
                        XulBulletinBoard bb = new XulBulletinBoard((Element)child, peerMap);

                        // add it
                        window.getContentPane().add(bb);

                        // add to peerMap
                        peerMap.put(child, bb);
                    }
                    */
                    else if (childElementName.equalsIgnoreCase(XulSyntax.ELEMENT_SCRIPT) ||
                             childElementName.equalsIgnoreCase(XulSyntax.ELEMENT_HTMLSCRIPT))
                    {

                        // should have a child text node
			String script = null;
			File scriptfile = null;

			//A context is related to the current thread. Therefore, the
			//jsEngine can't create it! We need to! Note that multiple
			//files can be loaded from one XUL file.

			jsEngine = jsEngine.getInstance();

			System.out.println("about to handle script for child=" + child.getNodeName());
			if (child.hasChildNodes())
			{
                                NodeList scriptChildren = child.getChildNodes();
                          	org.w3c.dom.Node first = scriptChildren.item(0);
				script = first.getNodeValue();
			}
			else
			{
				System.out.println("tag is " + child.getNodeValue());
				Attr srcAttr = ((Element)child).getAttributeNode("src");
				if (srcAttr != null)
					script = srcAttr.getNodeValue();
				System.out.println("attribute src is " + script);
				scriptfile = new File(script);
			}

			if (script != null)
			{
                               if (!jsEngine.setXulDom(xulDom))
                                    System.out.println("Cannot get a reference to the JS doc object.");

                                // add the script to the context
				if (scriptfile != null && scriptfile.exists())
					jsEngine.loadScriptFile(scriptfile);
				else
                                	jsEngine.loadScript(script);

                                if (debug)
                                	jsEngine.showIds();
                         }
                    }
                    else
                    {
                        System.out.println("*TBD*:Unhandled top-level child: " + childElementName);
                    }

                    /*
                    else if ()
                    {

                    }
                    */

                } // end of for (all children)
            } // end of if (root has child nodes)?

            window.addWindowListener(new WindowAdapter()
                                    {
                                        public void windowClosing(WindowEvent evt)
                                        {   System.exit(0); }
                                    });

            // if a styleSheet exists - apply it
            if (styleSheet != null)
                styleSheet.apply(xulDom);

            // have size?
            int w = window.getWidth();
            int h = window.getHeight();
            if (w <= 0 && h <= 0)
                window.pack();

            // show the window
            System.out.println("SHOWING THE WINDOW.");
            window.show();

            /* Not working ... Figure out why?
            if (styleSheet != null)
            {
                styleSheet.applyPositionalStyles(xulDom);
                window.repaint();
            }
            */

            // TBD: attach the event handlers

            // if an onLoad script - run it
            String windowOnLoad = window.getOnLoad();
            if (windowOnLoad != null)
            {
                // run it
                jsEngine.executeFunction(windowOnLoad, "windowOnLoad");
            }

	    //pretty sad, but thread can't finish until application's done
	    while (true)
	    {
		Thread.sleep(1000000000);
	    }

        } catch (Exception e)
          {
            e.printStackTrace();
          }
    }

    public void finalize()
    {
        // Exit from the context.
        //Context.exit();
    }

    /**
     * For unit testing.
     */
    public static void main(String args[])
    {
        try
        {

        } catch (Throwable t)
          {
            t.printStackTrace();
          }
    }
}
